@@ -12,6 +12,7 @@
 ## 2020-02-21 secr 4.2.0
 ## 2020-05-15 stringsAsFactors function
 ## 2020-07-14 secr 4.3.0
+## 2020-09-05 getknownclass factor bug fixed
 #######################################################################################
 
 # Global variables in namespace
@@ -20,8 +21,8 @@
 ## e.g. Roger Peng https://stat.ethz.ch/pipermail/r-devel/2009-March/052883.html
 
 .localstuff <- new.env()
-##.localstuff$packageType <- ' pre-release'
-.localstuff$packageType <- ''
+.localstuff$packageType <- ' pre-release'
+## .localstuff$packageType <- ''
 .localstuff$validdetectors <- c('single','multi','proximity','count', 
     'polygonX', 'transectX', 'signal', 'polygon', 'transect', 
     'capped', 'null','null','null','null', 'telemetry', 'signalnoise')
@@ -1161,11 +1162,14 @@ getknownclass <- function(capthist, nmix, hcov) {
     }
     else {
         if ((nmix>1) & (!is.null(hcov))) {
-            tmp <- as.numeric(factor(covariates(capthist)[,hcov])) + 1
-            tmp[is.na(tmp) | (tmp>(nmix+1))] <- 1
-            attr(tmp,'levels') <- levels(factor(covariates(capthist)
-                                                [,hcov]))[1:nmix]
-            tmp
+          ## 2020-09-05 use as.factor() instead of factor() to coerce 
+          ## (if already factor, coercing with factor() loses old levels)
+          var <- as.factor(covariates(capthist)[,hcov])
+          tmp <- as.numeric(var) + 1
+          tmp[is.na(tmp) | (tmp>(nmix+1))] <- 1
+          attr(tmp,'levels') <- levels(factor(covariates(capthist)
+            [,hcov]))[1:nmix]
+          tmp
         }
         else
             rep(1,nrow(capthist))
@@ -1939,7 +1943,7 @@ outsidemask <- function(CH, mask, threshold = spacing(mask) / sqrt(2)) {
 }
 ############################################################################################
 
-shareFactorLevels <- function (object, columns = NULL, stringsAsFactors = FALSE) {
+shareFactorLevels <- function (object, columns = NULL, stringsAsFactors = TRUE) {
     ## stringsAsFactors added 2020-05-16
     if (ms(object)) {
         if (!is.null(covariates(object))) {
@@ -1948,7 +1952,7 @@ shareFactorLevels <- function (object, columns = NULL, stringsAsFactors = FALSE)
                 columns <- 1:ncol(df)
             }
             if (stringsAsFactors) {
-                df[,columns] <- stringsAsFactors(df[,columns])
+                df[,columns] <- stringsAsFactors(df[,columns, drop = FALSE])
             }
             for (i in columns) {
                 if (is.factor(df[,i])) {
@@ -1967,7 +1971,7 @@ shareFactorLevels <- function (object, columns = NULL, stringsAsFactors = FALSE)
             if (is.null(columns)) {
                 columns <- 1:ncol(object)
             }
-            object[,columns] <- stringsAsFactors(object[,columns])
+            object[,columns] <- stringsAsFactors(object[,columns, drop = FALSE])
         }
     }
     object
